/*
 *
 * X DLX Simulator
 *
 * The DLX is a 32-bit RISC machine based on the MIPS 2000 architecture.
 * The architecture includes 32 32-bit general purpose registers and a 
 * program counter.
 *
 * Floating point registers and an interrupt register are not implemented.
 *
 * The address space is 4 Gbytes, big-endian byte-addressable using 32-bit
 * addresses. All instructions must be properly aligned.
 *
 *
 * Cameron Humphries
 * Department of Computer Science
 * University of Adelaide
 * 1992
 *
 */
 
#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/Xaw/Form.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/filio.h>

#include "sim.h"
#include "xdlx.bm"

#define NO_INPUT 0
#define INPUT_OK 1

Widget toplevel, form;
XtAppContext app_context;

void Create_File_Menu(Widget);
void Create_Execute_Menu(Widget);
void Create_BrkPt_Menu(Widget);
void Create_Code_View(Widget);
void Create_Register_View(Widget);
void Create_Data_View(Widget);
void Create_IO_View(Widget);
void Create_Status_View(Widget);
void Read_Comm_Channel(XtPointer, int *,XtInputId *);
void Update_all(),Write_Comm_Channel_Str(),Write_Comm_Channel_Int();

extern void Write_Status_Message();
extern int input_allowed;
extern int current_code_selection;

int num_kb=MEM_SIZE/1024;
int comms_sock[2];
int sim_pid;
char *dlx_filename = NULL;
int num_output_mods = 0;

main(int argc, char **argv)
{
  XWMHints *xwmhints;
  
  if (socketpair(AF_UNIX,SOCK_STREAM,0,comms_sock))
    {
      perror("socketpai");
      exit(1);
    }

  init(num_kb*1024);
  
  if (argc == 2)
    {
      dlx_filename = (char *)malloc(strlen(argv[1])+1);
      strcpy(dlx_filename,argv[1]);
      load(dlx_filename);
    }

  if (!(sim_pid=fork()))
    {
      sim_main(1);
    }

  /* Create initial X server connection */
  toplevel = XtAppInitialize(&app_context,"Xdlx",NULL,0,&argc,argv,
			     NULL,NULL,0);
			     
  /* Create geometry management widget */
  form = XtCreateManagedWidget("form",formWidgetClass,toplevel,NULL,0);

  /* Create menus */
  Create_File_Menu(form);
  Create_Execute_Menu(form);
  Create_BrkPt_Menu(form);
  
  /* Create Code view */
  Create_Code_View(form);
  
  /* Create Registers */
  Create_Register_View(form);
  
  /* Create Data view */
  Create_Data_View(form);
  
  /* Create I/O window */
  Create_IO_View(form);
  
  /* Create Status Line */
  Create_Status_View(form);
  
  /* Place windows on screen */
  XtRealizeWidget(toplevel);

  xwmhints = XAllocWMHints();
  xwmhints->icon_pixmap = XCreateBitmapFromData(XtDisplay(toplevel),XtWindow(toplevel),
						xdlx_bits,xdlx_width,xdlx_height);
  xwmhints->flags |= IconPixmapHint;
  XSetWMHints(XtDisplay(toplevel),XtWindow(toplevel),xwmhints);

  
  Write_Status_Message("READY");

  /* Start event processing */
  XtAppAddInput(app_context,comms_sock[0],XtInputReadMask,Read_Comm_Channel,NULL);
  XtAppMainLoop(app_context);
  
  exit(0);
}

void Read_Comm_Channel(XtPointer client_data, int *source,XtInputId *id)
{
  char buf[1024];
  char mesg_type[4];
  unsigned int mesg_length;
  void Change_Text_Cursor();
  int num_pending;
  
  read(*source,mesg_type,1);
  switch (mesg_type[0])
    {
    case 's':
      Update_all(); 
      Display_Output();
      num_output_mods = 0;
      break;
    case 'm':
      read(*source,&mesg_length,4);
      read(*source,buf,mesg_length);
      Write_Status_Message(buf);
      Display_Output();
      num_output_mods = 0;
      break;
    case 'i':
      input_allowed = INPUT_OK;
      /* Change cursor to hand2 */
      Change_Text_Cursor();
      Write_Status_Message("Expecting Input");
      Display_Output();
      num_output_mods = 0;
      break;
    case 'o':
      read(*source,mesg_type,1);
      Prepare_Output(mesg_type[0]);
      num_output_mods++;
      if ((!ioctl(*source,FIONREAD,&num_pending)) && 
	  num_pending > 1 && num_output_mods > 200)
	{
	  Display_Output();
	  num_output_mods = 0;
	}
      break;
    }
  
  return;
}

void Write_Comm_Channel_Str(char *mesg)
{
  write(comms_sock[0],mesg,strlen(mesg));
  
  return;
}

void Write_Comm_Channel_Int(int mesg)
{
  write(comms_sock[0],&mesg,sizeof(int));
  
  return;
}

void Update_all()
{
  /* Update reg values including PC */
  Update_Regs();
  /* Update code view */
  Update_Code();
  /* Update data view */
  Update_Data();
  
  return;
}
